@@ -9,6 +9,8 @@
 struct EMD {
     static float distance(const MixtureModel &m1, const MixtureModel &m2);
     static float distance(float beats1[BEATSSIZE], float beats2[BEATSSIZE]);
+    static float raw_distance(const MixtureModel &m1, const MixtureModel &m2);
+    static float raw_distance(float beats1[BEATSSIZE], float beats2[BEATSSIZE]);
 private:
     static float gauss_dist(int *f1, int *f2)
         { return cost[*f1][*f2]; }
