@@ -3,16 +3,9 @@
 
 #include "immsconf.h"
 #include "immsdb.h"
-#include "songinfo.h"
 #include "song.h"
 
-#define     NEW_PLAYS               2
-#define     NEW_BONUS               20
-
-#define     TREND_FACTOR            3.0
-#define     MAX_TREND               20
-
-class InfoFetcher : virtual protected ImmsDb, private SongInfo
+class InfoFetcher : virtual protected ImmsDb
 {
 public:
     InfoFetcher() : next_sid(-1) {}
@@ -27,15 +20,15 @@ protected:
         void reset();
         bool get_song_from_playlist();
 
-        int position, rating, composite_rating;
-        int relation, bpmrating, specrating, newness;
+        Rating rating;
+        int position, effective_rating;
+        int relation, bpmrating, specrating;
         time_t last_played;
         bool identified;
-        float trend_scale;
     };
 
     virtual bool fetch_song_info(SongData &data);
-    virtual bool parse_song_info(const string &path, StringPair &info);
+    virtual bool parse_song_info(const SongData &data, StringPair &info);
 
     bool identify_playlist_item(int pos);
 
