@@ -8,6 +8,8 @@
 #include "picker.h"
 #include "xidle.h"
 #include "serverstub.h"
+#include <analyzer/mfcckeeper.h>
+#include <analyzer/beatkeeper.h>
 
 // IMMS, UMMS, we all MMS for XMMS?
 
@@ -26,10 +28,8 @@ public:
     void start_song(int position, std::string path);
     void end_song(bool at_the_end, bool jumped, bool bad);
 
-    // get the last song played
-    int  get_previous();
+    virtual void playlist_ready();
 
-    virtual void request_playlist_item(int index);
     void playlist_changed(int length);
 
     // process internal events - call this periodically
@@ -38,18 +38,26 @@ public:
     // configure imms
     void setup(bool use_xidle);
 
-    friend class ImmsDFilter;
+    void sync(bool incharge);
+
+    friend class ImmsProcessor;
 
 protected:
     struct LastInfo {
-        LastInfo() : sid(-1) {}
+        LastInfo() : sid(-1), avalid(false) {}
         time_t set_on;
-        int sid;
-        StringPair acoustic;
+        int uid, sid;
+        bool avalid;
+        MixtureModel mm;
+        float beats[BEATSSIZE];
     };
 
-    void reset_selection();
-    void get_metacandidates();
+    virtual void playlist_updated() { server->playlist_updated(); }
+
+    // Implementations for SongPicker
+    virtual void request_playlist_item(int index);
+    virtual void get_metacandidates(int size);
+    virtual void reset_selection();
 
     // Helper functions
     bool fetch_song_info(SongData &data);
