@@ -1,10 +1,12 @@
 #ifndef __SONGINFO_H
 #define __SONGINFO_H
 
-#include "immsconf.h"
+#include <time.h>
 
 #include <string>
 
+#include "immsconf.h"
+
 using std::string;
 
 class InfoSlave
@@ -13,6 +15,9 @@ public:
     virtual string get_artist() { return ""; }
     virtual string get_title()  { return ""; }
     virtual string get_album()  { return ""; }
+
+    virtual time_t get_length() { return 0; }
+
     virtual ~InfoSlave() {};
 };
 
@@ -20,15 +25,13 @@ class SongInfo : public InfoSlave
 {
 public:
     SongInfo() : filename(""), myslave(0) { };
-    SongInfo(const string &_filename) { link(_filename); }
+    SongInfo(const string &_filename) : myslave(0) { link(_filename); }
     ~SongInfo() { delete myslave; }
 
-    virtual string get_artist()
-        { return myslave->get_artist(); }
-    virtual string get_title()
-        { return myslave->get_title(); }
-    virtual string get_album()
-        { return myslave->get_album(); }
+    virtual string get_artist();
+    virtual string get_title();
+    virtual string get_album();
+    virtual time_t get_length();
 
     void link(const string &_filename);
 
