@@ -1,5 +1,6 @@
 #include <sqlite3.h>
 #include <sstream>
+#include <time.h>
 
 #include "sqlite++.h"
 
@@ -35,7 +36,7 @@ void SQLDatabase::open(const string &filename)
     if (sqlite3_open(filename.c_str(), &db_ptr))
         throw SQLStandardException();
 
-    sqlite3_busy_timeout(db(), 5000);
+    sqlite3_busy_timeout(db(), 30000);
 }
 
 void SQLDatabase::close()
@@ -130,10 +131,21 @@ void AttachedDatabase::detach()
 
 // AutoTransaction
 
-AutoTransaction::AutoTransaction() : commited(false)
+AutoTransaction::AutoTransaction(bool exclusive) : commited(true)
 {
-    if (sqlite3_exec(SQLDatabase::db(), "BEGIN TRANSACTION;", 0, 0, 0))
-        commited = true;
+    if (!sqlite3_get_autocommit(SQLDatabase::db()))
+        return;
+
+    int r;
+    string exclusive_term = exclusive ? "EXCLUSIVE " : "";
+    string query = "BEGIN " + exclusive_term + " TRANSACTION;";
+    do { r = sqlite3_exec(SQLDatabase::db(), query.c_str(), 0, 0, 0); }
+    while (r == SQLITE_BUSY);
+
+    if (r)
+        throw SQLStandardException();
+
+    commited = false;
 }
 
 AutoTransaction::~AutoTransaction()
@@ -144,27 +156,41 @@ AutoTransaction::~AutoTransaction()
 
 void AutoTransaction::commit()
 {
-    if (!commited)
-        sqlite3_exec(SQLDatabase::db(), "COMMIT TRANSACTION;", 0, 0, 0);
+    if (commited)
+        return;
+
     commited = true;
+
+    while (1)
+    {
+        int r = sqlite3_exec(SQLDatabase::db(), "COMMIT TRANSACTION;", 0, 0, 0);
+        if (!r)
+            return;
+        if (r != SQLITE_BUSY)
+            throw SQLStandardException();
+#ifdef DEBUG
+        cerr << "database busy at commit time - sleeping" << endl;
+#endif
+        struct timespec t = { 0, 250000000 };
+        nanosleep(&t, 0);
+    }
 }
 
 // SQLQueryManager
 
 SQLQueryManager *SQLQueryManager::instance;
 
-sqlite3_stmt *SQLQueryManager::get(const string &query)
+sqlite3_stmt *SQLQueryManager::get(const string &query, bool &cached)
 {
     StmtMap::iterator i = statements.find(query);
 
     if (i != statements.end())
-        return i->second;
-
-#ifdef TRANS
-    int tr = 1;
-    if (cache)
-        tr = sqlite3_exec(SQLDatabase::db(), "BEGIN TRANSACTION;", 0, 0, 0);
-#endif
+    {
+        if (!sqlite3_expired(i->second))
+            return i->second;
+        sqlite3_finalize(i->second);
+        statements.erase(query);
+    }
 
     sqlite3_stmt *statement = 0;
     int qr = sqlite3_prepare(SQLDatabase::db(), query.c_str(),
@@ -172,12 +198,6 @@ sqlite3_stmt *SQLQueryManager::get(const string &query)
 
     SQLException except = SQLStandardException();
 
-#ifdef TRANS
-    if (tr == 0) 
-        if (sqlite3_exec(SQLDatabase::db(), "COMMIT TRANSACTION;", 0, 0, 0))
-            throw SQLStandardException();
-#endif
-
     if (qr)
     {
         if (block)
@@ -185,6 +205,7 @@ sqlite3_stmt *SQLQueryManager::get(const string &query)
         throw except;
     }
 
+    cached = cache;
     if (cache)
         statements[query] = statement;
     return statement;
@@ -211,9 +232,17 @@ SQLQueryManager::~SQLQueryManager()
 
 // SQLQuery
 
-SQLQuery::SQLQuery(const string &query) : curbind(0), stmt(0)
+SQLQuery::SQLQuery(const string &query) : curbind(0), cached(true), stmt(0)
 {
-    stmt = SQLQueryManager::self()->get(query);
+    stmt = SQLQueryManager::self()->get(query, cached);
+}
+
+SQLQuery::~SQLQuery()
+{
+    reset();
+
+    if (stmt && !cached)
+        sqlite3_finalize(stmt);
 }
 
 bool SQLQuery::next()
@@ -231,6 +260,13 @@ bool SQLQuery::next()
     return false;
 }
 
+bool SQLQuery::is_null()
+{
+    if (!stmt)
+        return false;
+    return sqlite3_column_type(stmt, curbind) == SQLITE_NULL;
+}
+
 void SQLQuery::reset()
 {
     curbind = 0;
@@ -280,6 +316,14 @@ SQLQuery &SQLQuery::operator<<(const string &s)
     return *this;
 }
 
+SQLQuery &SQLQuery::bind(const void *data, size_t n)
+{
+    if (stmt)
+        if (sqlite3_bind_blob(stmt, ++curbind, data, n, SQLITE_TRANSIENT))
+            throw SQLStandardException();
+    return *this;
+}
+
 SQLQuery &SQLQuery::operator<<(const SQLExec &execute)
 {
     this->execute();
@@ -324,6 +368,21 @@ SQLQuery &SQLQuery::operator>>(double &i)
     return *this;
 }
 
+SQLQuery &SQLQuery::load(void *data, size_t &n)
+{
+    if (!stmt)
+        return *this;
+
+    size_t blobsize = sqlite3_column_bytes(stmt, curbind);
+    if (blobsize > n)
+        throw SQLException("Error",
+                "insufficient memory provided for load BLOB");
+    n = blobsize;
+    const void *blobdata = sqlite3_column_blob(stmt, curbind++);
+    memcpy(data, blobdata, blobsize);
+    return *this;
+}
+
 SQLQuery &SQLQuery::operator>>(float &i)
 {
     double j = 0;
